#!/usr/bin/python

import os
import re
import sys
import shutil
import tempfile
from shlex import split
from subprocess import (
    call,
    check_call,
    check_output
)

sys.path.insert(0, os.path.join(os.environ['CHARM_DIR'], 'lib'))

from charmhelpers.core import (
    hookenv,
)
log = hookenv.log

def add_to_path(p, new):
    return p if new in p.split(':') else p + ':' + new

os.environ['PATH'] = add_to_path(os.environ['PATH'], '/usr/lpp/mmfs/bin')

## ssh key methods ##
def create_ssh_keys():
    # Generate ssh keys if needed
    log("Configuring ssh keys")
    if not os.path.isfile("/root/.ssh/id_rsa"):
        call(split('ssh-keygen -q -N "" -f /root/.ssh/id_rsa'))
        # Ensure permissions are good
        os.chmod('/root/.ssh/id_rsa.pub', 0600)
        os.chmod('/root/.ssh/id_rsa', 0600)
   
def get_ssh_keys():
    # get public and private ssh keys
    with open("/root/.ssh/id_rsa.pub", "r") as idfile:
        pubkey = idfile.read()
        idfile.close()
    with open("/root/.ssh/id_rsa", "r") as idfile:
        privkey = idfile.read()
        idfile.close()
    with open("/root/.ssh/authorized_keys", "w+") as idfile:
        idfile.write(pubkey)
        idfile.close()
    return [privkey, pubkey]

def set_ssh_key(key, private=False):
    if key:
        if private == False:
            f = open("/root/.ssh/id_rsa.pub", "w")
            f.write(key)
            f.close()
            os.chmod('/root/.ssh/id_rsa.pub', 0600)

            with open("/root/.ssh/authorized_keys", "w+") as idfile:
                idfile.write(key)
                idfile.close()
        else:
            f = open("/root/.ssh/id_rsa", "w")
            f.write(key)
            f.close()
            os.chmod('/root/.ssh/id_rsa', 0600)

def configure_ssh():
    # Configure sshd_config file to allow root
    sshconf = open("/etc/ssh/sshd_config", 'r')
    tf = tempfile.NamedTemporaryFile(mode='w+t', delete=False)
    tfn = tf.name
    for line in sshconf:
        if not line.startswith("#"):
            if "PermitRootLogin" in line:
                tf.write("# Updated by GPFS charm: ")
                tf.write(line)
            else:
                tf.write(line)
        else:
            tf.write(line)
    tf.write("# added by GPFS charm:\n")
    tf.write("PermitRootLogin without-password\n")
    sshconf.close()
    tf.close()
    shutil.copy(tfn, '/etc/ssh/sshd_config')
    call(split('service ssh reload'))

    # Avoid the host key confirmation
    with open("/root/.ssh/config", "w+") as idfile:
        idfile.write("StrictHostKeyChecking no\n")
        idfile.close()

## cluster management methods ##

def start():
    # Only manager can start/stop cluster
    if is_manager():
        log(check_output(split('mmstartup -a')))

def stop():
    # stop the server
    if is_manager():
        check_output(split('mmshutdown -a'))

def node_exists(nodename):
    # Check if node has already been added to cluster
    lscluster = check_output('mmlscluster')
    node = re.search('^.*\d+.*%s.*\n' % nodename, lscluster, re.M)
    return False if node is None else True

def add_node(nodename, m_designation='client', q_designation='nonquorum'):
    # add new node to the cluster
    if cluster_exists() and not node_exists(nodename):
        log(check_output(split('mmaddnode -N %s:%s-%s' % \
                    (nodename, m_designation, q_designation))))
    else:
        log('Node %s could not be added. Manager not defined' % nodename)
  
def apply_license(nodename):
    check_output(split('mmchlicense server --accept -N %s' % nodename))

def set_manager_file():
    # define a file that indicates the node is a manager
    open("/var/mmfs/.manager","a").close()

def is_manager():
    # check if the node is a manager
    if os.path.exists("/var/mmfs/.manager"):
        return True

def cluster_exists():
    # Check if cluster is already defined
    return True if call('mmlscluster') == 0 else False
       
def create_cluster(hostname):
    # create the cluster for the hostname, manager only
    if not is_manager() and not cluster_exists():
        log(check_output(split('mmcrcluster -N %s:quorum-manager' % \
                    hostname + ' -r /usr/bin/ssh -R /usr/bin/scp')))
